//------------------------------------------------------------------------------
//
//  npglut.h
//  npe
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------

#ifndef _NPTYPES_H
#define _NPTYPES_H


#include <freeglut.h>
#include "stdbool.h"


#define kPI			 3.141593f
#define kPos2PI		 6.283185f
#define kNeg2PI		-6.283185f

#define kKeyMapSize 256
#define kKeyEventTypeSize 4

#define	kPlotMax 16384
#define kPlotChildMax 32
#define kGridCount 10
#define kDefaultScale = 1.0f
#define kColorIndexSize 20



// OpenGL generic use ---------------------------------------------------------
typedef struct {
   GLdouble x,y,z;
} recVec;

typedef struct {
	GLfloat angle,x,y,z;
} GLfloatAXYZ;

typedef struct {
	GLfloat r,g,b,a;
} GLfloatRGBA;

typedef struct {
	GLfloat r,g,b,a;
} GLfloatRGB;

typedef struct {
	int x,y,z;
} IntXYZ;

typedef struct {
	GLubyte r,g,b,a;
} GLubyteRGBA;

typedef struct {
	GLboolean x,y,z;
} BoolXYZ;

typedef struct {
	GLboolean angle,x,y,z;
} BoolAXYZ;

struct GLfloatXY {
	GLfloat x,y;
};
typedef struct GLfloatXY GLfloatXY;
typedef struct GLfloatXY *GLfloatXYPtr;

struct GLfloatXYZ {
	GLfloat x,y,z;
};
typedef struct GLfloatXYZ GLfloatXYZ;
typedef struct GLfloatXYZ *GLfloatXYZPtr;


// Global Data Structure ------------------------------------------------------

struct Key {
	int		modShift;
	int		modCtrl;
	int		modAlt;
	int		modCommand;
	int		map[kKeyEventTypeSize][kKeyMapSize];
};
typedef struct Key Key;
typedef struct Key * pKey;

struct DataMap {
	int		size;					// memory used, add/del should modify this, debug zz

	void*	plot[kPlotMax];
	int		plotCount;
	int		plotIndex;

	void*	activePlot;				//active plot for traversing tree
	void*	selectedPlot;			//current plot selection, can be a child branch
	int		selectedIndex;			//helpful to know which tree we are on

	BoolXYZ	selectionSet;			//current selection set to add too
	int		selectAll;				//true when all plots selected
};
typedef struct DataMap DataMap;
typedef struct DataMap * pDataMap;

struct IO {
	Key		key;
	int		channelCount;
	int		drawMenu;
	int		clearFrameBuffer;
	BoolXYZ	axes;					//should this be moved to Ctrl?			debug, zz
	int		size;

	GLfloatXYZ	camTranslate[4];
	GLfloatXYZ	camRotate[4];
	int			camIndex;
};
typedef struct IO IO;
typedef struct IO * pIO;

struct Ctrl {
	int		size;
};
typedef struct Ctrl Ctrl;
typedef struct Ctrl * pCtrl;

struct Data {
	DataMap map;
	IO		io;
	Ctrl	ctrl;
};
typedef struct Data Data;
typedef struct Data * pData;


// plots are organized as a branched tree, supports complexity of linked objects
// ie a recPlot of type plotCamera may have a multiple depth daughter tree that
// supports several cameras or videoPlots for circle cam rigs etc...
//-----------------------------------------------------------------------------
struct NPplot
{
	int			type;						//plot type, point, pin, cam, video
	void*		data;						//plot type specific data
	
	int			selected;					//true if plot currently selected

	void*		parent;						//parent plot, binary tree of plots
	int			branchLevel;				//0 is the trunk, 1 is 1st branch,
											// 2 is branch off of 1st branch
	void*		child[kPlotChildMax];		//children attached to this one
	int			childIndex;					//the currently selected child
	int			childCount;					//current number of children

	IntXYZ		channel;					//channel to use for x,y,z points
	IntXYZ		channelIndex;				//last updated channel index

	IntXYZ		averageType;				//averaging type applied to data
	IntXYZ		sampleInterval;				//the interval to be averaged

	GLfloatAXYZ	rotate;						//orientation
	GLfloatXYZ	scale;						//scale plot XYZ
	GLfloatXYZ	translate;					//location relative to plotOrigin
	GLfloatXYZ	plotOrigin;					//origin relative to parent origin

	GLfloatXYZ	rotateRate;					//rotation rate
	GLfloatXYZ	rotateRad;					//accumalates the rotation angles in radians
	GLfloatXYZ	scaleRate;					//scale rate 
	GLfloatXYZ	translateRate;				//pans the location of the plot, pronunciar en Espanol, lol
	GLfloatXYZ	translateVect;				//pans the location of the plot, pronunciar en Espanol, lol

	int			surfaceType;				//FFT layers, mesh, sphere...
	int			shaderType;					//wire, flat, gouraud, phong

	GLfloat		lineWidth;					//draws lines, zero for off
	GLfloat		pointSize;					//draws dots, zero for off
	GLfloat		circle;						//draw circle at current data point

	int			colorIndex;
	GLubyteRGBA	color;						//color assigned to new data
	GLboolean	colorFade;

	GLboolean	hide;						//hide the plot, data stays active
	GLboolean	freeze;						//freezes data and physics updates
	GLboolean	center;						//centers to plotOrigin
	GLboolean	centerPerGrid;				//centers in gridSpace increments

	BoolXYZ		autoZoom;					//scales plot to fit screen
	GLboolean	scroll;						//scroll right to left
	
	BoolXYZ		triggerOnHi;				//triggers are multipurpose
	BoolXYZ		triggerOnLow;
	GLfloatXYZ	triggerLevelHi;
	GLfloatXYZ	triggerLevelLow;
	int			triggerCommand;				//execute ctrlCommand upon trigger
	
	BoolXYZ		proximity;					//used for collision engine 
	GLfloatXYZ	proximityRadius;

	BoolXYZ		drawCurrentValue;			//draw current values as text
	BoolXYZ		drawTranslate;				//draw translate as text
	BoolXYZ		drawScale;					//draw scale as text
	BoolAXYZ	drawRotate;					//draw rotation as text

	BoolXYZ		selectionSet;				//belongs to selection set
};
typedef struct NPplot NPplot;
typedef struct NPplot *NPplotPtr;


// cameras can be GL scene cameras or real-world cameras
// video is attached as a daughter of the recPlot
//-----------------------------------------------------------------------------
struct NPcamera
{
	int			textureID;					//texture ID in GL

	int			format;						//320p, 480i, 720p, 4K full app
	int			interlaced;					//interlaced field order
	int			stereo;						//stereoscopic 3D
	GLfloat		aspectRatio;				//1.0, 1.333, 1.777, 1.85, 2.25... 
	GLfloat		fps;						// 15, 24, 29.97, 30, 59.94, 120...
	int			colorSpace;					//8, 12bit, YUV, RGBA, XYZ, CMYK...
	int			width;						//res in pixels
	int			height;

	GLfloat		fov;						//FOV 35mm, 70mm...
	GLfloat		clipNear;
	GLfloat		clipFar;

	GLfloat		aperture;					//F stop
	GLfloat		exposure;					//in seconds
	int			sensorType;					//3CCD, Debayer pattern...
};
typedef struct NPcamera NPcamera;
typedef struct NPcamera *NPcameraPtr;


//-----------------------------------------------------------------------------
struct NPvideo
{
	int			textureID;					//texture ID in GL

	int			format;						//320p, 480i, 720p, 4K full app
	int			interlaced;					//interlaced field order
	int			stereo;						//stereoscopic 3D
	GLfloat		aspectRatio;				//1.0, 1.333, 1.777, 1.85, 2.25... 
	GLfloat		fps;						// 15, 24, 29.97, 30, 59.94, 120...
	int			colorSpace;					//8, 12bit, YUV, RGBA, XYZ, CMYK...
	int			width;						//res in pixels
	int			height;

	GLfloatRGBA	bias;						//color shift
	GLfloatRGBA	gain;						//color multiplier
};
typedef struct NPvideo NPvideo;
typedef struct NPvideo *NPvideoPtr;


//-----------------------------------------------------------------------------
struct NPsurface
{
	int			textureID;					//texture assigned to element

	GLfloat		*vertexArray;				//the array of vertices
	GLfloat		*normalArray;				//the array of normals
	GLubyte		*colorArray;				//each point gets color assigned
	int			vertexCount;				//
	int			normalCount;				//
	int			arrayIndex;					//current vertex index in the array
};
typedef struct NPsurface NPsurface;
typedef struct NPsurface *NPsurfacePtr;


//-----------------------------------------------------------------------------
struct NPpoints
{
	GLfloat		*vertexArray;				//the array of data points
	GLubyte		*colorArray;				//each point gets color assigned
	int			vertexCount;				//number of points in the plot
	int			arrayIndex;					//current vertex index in the array
};
typedef struct NPpoints NPpoints;
typedef struct NPpoints *NPpointsPtr;


//-----------------------------------------------------------------------------
struct NPpin
{
	int			textureID;					//texture assigned to element

	GLfloat		innerRadius;				//inner radius of the toroid
	GLfloat		outerRadius;				//outer radius of the toroid

	int			radiusRatioIndex;			//sorts pre-loaded GPU toriods
	int			screenSizeIndex;			//same as above for detail level
	
	int			slices;						//calculated from radiusRatioIndex
	int			stacks;						//...and from the screenSizeIndex

	GLfloatXYZ	translateTex;
	GLfloatXYZ	scaleTex;
	GLfloatAXYZ	rotateTex;					//orientation of the texture map
};
typedef struct NPpin NPpin;
typedef struct NPpin *NPpinPtr;


//-----------------------------------------------------------------------------
struct NPgrid
{
	IntXYZ		segments;
	GLfloatXYZ	size;
	GLboolean	overlay;					//overlays relative to screen coord
};
typedef struct NPgrid NPgrid;
typedef struct NPgrid * NPgridPtr;


// constants ------------------------------------------------------------------
enum
{	
	kPlotDefault,			//simplest type, no plot data
	kPlotCamera,			//for both real-world and virtual GL cameras 
	kPlotVideo,				//video from live/stream, internal GL, or file
	kPlotSurface,			//ground, sky... background objects drawn first
	kPlotPoints,			//for X/XY/XYZ line, eeg/ecg, temp, movement...
	kPlotPin,				//combo of cones, spheres, toroids, etc...
	kPlotPinTorus,			//child of a PlotPin
	kPlotGrid				//grid in 1D, 2D or 3D
};

enum
{
	kSurfaceNull = 0,
	kSurfacePlane,
	kSurfaceCylinder,
	kSurfaceCone,
	kSurfaceSphere,
	kSurfaceTorus,
	kSurfacePin,							// ice-cream cone shape
	kSurfaceMesh,							// terrain...
	kSurfaceLayers,							// ie stack of FFT images, MRI...
	kSurfaceCamera,
	kSurfaceTypeCount
};

enum
{
	kSelectNULL = 0,
	kSelectAll,
	kSelectNone,
	kSelectAdd,
	kSelectRemove,
	kSelectColor,
	kSelectParent,
	kSelectSiblings,
	kSelectChildren,
	kSelectProperty
};

enum
{
	kGetNull = 0,
	kNodeColor,
	kNodeChannel,
	kNodeID
};

enum
{
	kMapTypeNull = 0,
	
	kMapTypeAll,

	kMapTypeUniverses,
	kMapTypeUniverse,
	kMapTypeGalaxyCluster,
	kMapTypeGalaxy,
	kMapTypeSolar,

	kMapTypeGlobal,				// Earth
	kMapTypeRegion,				// < 30 ms latency, < 4000km
	kMapTypeLocation,			// < 3 ms latency, < 400km
	kMapTypeCluster,			// < 1 ms latency, < 40km
	kMapTypeNode,				// < 1 us latency, < 4m
						
	kMapTypeBUS,				// Node maps
	kMapTypeCPU,
	kMapTypeRAM,
	kMapTypeGPU,

	kMapTypeIO,					// typically a bus device if a node

	kMapTypeDrive,
	kMapTypeNetwork,
	kMapTypeVideo,
	kMapTypeAudio,

	kMapTypeHaptic,
	kMapTypeKey,
	kMapTypeMouse,
	kMapTypeJoystick,
	kMapTypeTablet,
	kMapTypeTouchScreen,
	kMapTypeHID,

	kMapTypeArray,
	kMapTypeTree,
	kMapTypeNPE,
	kMapTypeDraw,
	kMapTypeZ,

	kMapTypeCount
};

enum
{
	kShadingNull = 0,
	kShadingWire,
	kShadingFlat,
	kShadingGouraud,
	kShadingPhong,
	kShadingReflection,
	kShadingRaytrace
};

enum
{
	kCameraSensorNull = 0,					//don't know the sensor type
	kCameraSensorMonoCCD,					//monochrome CCD such as BW, IR
	kCameraSensorDebayerCCD,				//single color CCD debayer pattern
	kCameraSensorThreeCCD,					//3CCD color sensor
	kCameraSensorDepthCCD					//BW or color with Z depth data
};

enum
{
	kVideoFormatNull = 0,
	kVideoFormatNTSC,
	kVideoFormatPAL,
	kVideoFormat720p,
	kVideoFormat1080i,
	kVideoFormat1080p,
	kVideoFormat2K,
	kVideoFormat4k,
	kVideoFormat8k,
	kVideoFormat16k,
	kVideoFormat32k,
	kVideoFormat64k
};

enum
{
	kVideoFieldNull = 0,
	kVideoFieldOdd,
	kVideoFieldEven
};

enum
{
	kVideoStereoNull,
	kVideoStereoLeft,
	kVideoStereoRight,
	kVideoStereoSequential
};

enum
{
	kColorSpaceNull = 0,
	kColorSpaceRGB_8,
	kColorSpaceRGB_10,
	kColorSpaceRGB_12,
	kColorSpaceRGB_16
};

enum {
	kAverageNull = 0,
	kAverage,
	kAverageRootMeanSquare,
	kAverageSmoothed,
	kAverageRunning
};

enum {
	kLeftJustify = 0,
	kRightJustify,
	kCenterJustify
};

enum {
	kSmallFont = 0,
	kMediumFont,
	kLargeFont
};

enum
{
	kChannelNull = 0,
	kChannelCycleCount,
	kChannelOscilloscope
};



// 120Hz timer, 60, 30, 24
// 119.88Hz timer, 59.94, 29.97, 23.976


//---- Graph Command Constants ----

enum {
	//global graph commands
	kCtrlCommandNull = 0,
	kGraphMenu = 4242,		//
	kGraphDrawIndicators,
	
	kGraphNew,
	kGraphDelete,

	kGraphNext,
	kGraphPrevious,
	kGraphNextBranch,
	kGraphPreviousBranch,

	kGraphGrid,
	kGraphCamera,


	kGraphSelect,
	kGraphSelectAll,
	kGraphSelectNone,
	kGraphSelectOn,
	kGraphSelectOff,
	kGraphSelectOne,
	kGraphSelectTwo,
	kGraphSelectThree,


	kGraphPresetOne,
	kGraphPresetTwo,
	kGraphPresetThree,
	kGraphPresetFour,
	kGraphPresetFive,
	kGraphPresetSix,
	kGraphPresetSeven,
	kGraphPresetEight,

	//translate, rotate and scale selected objects
	kGraphSelectAxes,
	
	kGraphXincrease,
	kGraphXincreaseOff,
	kGraphXdecrease,
	kGraphXdecreaseOff,
	
	kGraphYincrease,
	kGraphYincreaseOff,
	kGraphYdecrease,
	kGraphYdecreaseOff,
	
	kGraphZincrease,
	kGraphZincreaseOff,
	kGraphZdecrease,
	kGraphZdecreaseOff,
	
	kGraphRotateLeft,
	kGraphRotateLeftOff,
	kGraphRotateRight,
	kGraphRotateRightOff,

	kGraphRotateUp,
	kGraphRotateUpOff,
	kGraphRotateDown,
	kGraphRotateDownOff,

	kGraphRotateCCW,
	kGraphRotateCCWOff,
	kGraphRotateCW,
	kGraphRotateCWOff,

	kGraphZoomOn,
	kGraphZoomOff,
	
	kGraphClearFrameBuffer, 
	
	//plot specific graph commands
	kGraphSampleInterval,
	kGraphChannelDown,
	kGraphChannelUp,
	kGraphColorDown,
	kGraphColorFade,
	kGraphAltColor,
	kGraphColorUp,
	kGraphColorPallete,
	
	kGraphFreeze,
	kGraphHide,
	kGraphClear,
	
	kGraphCenter,
	kGraphScroll,

	kGraphPoints,	
	kGraphLines,
	kGraphSurface,
	kGraphMesh,

	kGraphAlphaUp,
	kGraphAlphaDown,
	kGraphBiasUp,
	kGraphBiasDown,
	kGraphGainUp,
	kGraphGainDown,

	kEventLowSetpoint,											// add kGraph..,  debug zz
	kEventHighSetpoint,

	kFullScreenExit
};



enum {	kKeyDown = 1,
		kKeyRepeat,
		kKeyUp,
		kKeyModifier
};

enum {	kGlutKeyDown = 1,
		kGlutKeyUp,
		kGlutKeyDownSpecial,
		kGlutKeyUpSpecial
};

enum {		kKeyCodeA = 0x00,
				kKeyCodeS,
				kKeyCodeD,
				kKeyCodeF,
				kKeyCodeH,
				kKeyCodeG,
				kKeyCodeZ,
				kKeyCodeX,
				kKeyCodeC,
				kKeyCodeV = 0x09,
				kKeyCodeB = 0x0b,
				kKeyCodeQ,
				kKeyCodeW,
				kKeyCodeE,
				kKeyCodeR,
				kKeyCodeY,
				kKeyCodeT,
				kKeyCode1,
				kKeyCode2,
				kKeyCode3,
				kKeyCode4,
				kKeyCode6,
				kKeyCode5,
				kKeyCodeEqual,
				kKeyCode9,
				kKeyCode7,
				kKeyCodeMinus,
				kKeyCode8,
				kKeyCodeZero,
				kKeyCodeRightBracket,
				kKeyCodeO,
				kKeyCodeU,
				kKeyCodeLeftBracket,
				kKeyCodeI,
				kKeyCodeP,
				kKeyCodeReturn,
				kKeyCodeL,
				kKeyCodeJ,
				kKeyCodeRightQuote,
				kKeyCodeK,
				kKeyCodeSemiColon,
				kKeyCodeBackSlash,
				kKeyCodeComma,
				kKeyCodeSlash,
				kKeyCodeN,
				kKeyCodeM,
				kKeyCodePeriod,
				kKeyCodeTab,
				kKeyCodeSpace,
				kKeyCodeTilda,
				kKeyCodeDelete = 0x33,
				kKeyCodeBackSpace,
				kKeyCodeESC = 0x35,
				
				kKeyCodeF1 = 0x7A,
				kKeyCodeF2 = 0x78,
				kKeyCodeF3 = 0x63,
				kKeyCodeF4 = 0x76,
				kKeyCodeF5 = 0x60,
				kKeyCodeF6 = 0x61,
				kKeyCodeF7 = 0x62,
				kKeyCodeF8 = 0x64,
				kKeyCodeF13 = 0x69,
				kKeyCodeF16 = 0x6A,
				
				kKeyCodeHelp = 0x72,
				kKeyCodeHome = 0x73,
				kKeyCodePageUp = 0x74,
				kKeyCodeDeleteRight = 0x75,
				kKeyCodeEnd = 0x77,
				kKeyCodePageDown = 0x79,
				
				kKeyCodeLeft = 0x7B,
				kKeyCodeRight = 0x7C, 
				kKeyCodeDown = 0x7D,
				kKeyCodeUp = 0x7E,
				
				kKeyCodeNumPadPeriod = 0x41,
				kKeyCodeNumPadStar = 0x43,
				kKeyCodeNumPadPlus = 0x45,
				kKeyCodeNumPadClear = 0x47, 
				kKeyCodeNumPadEnter = 0x4C,
				kKeyCodeNumPadSlash = 0x4B,
				kKeyCodeNumPadMinus = 0x4E,
				kKeyCodeNumPadEqual = 0x51,
				kKeyCodeNumPadZero = 0x52,
				kKeyCodeNumPad1 = 0x53,
				kKeyCodeNumPad2 = 0x54,
				kKeyCodeNumPad3 = 0x55,
				kKeyCodeNumPad4 = 0x56,
				kKeyCodeNumPad5 = 0x57,
				kKeyCodeNumPad6 = 0x58,
				kKeyCodeNumPad7 = 0x59,
				kKeyCodeNumPad8 = 0x5B,
				kKeyCodeNumPad9 = 0x5C
};

#endif
